package com.muyu.group.domain.model;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * @program: cloud-server
 * @description: 团购活动价格模型
 * @author: MengFanKuo
 * @create: 2024-11-21 15:28
 * *
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@SuperBuilder
public class TeamDiscountPriceModel {
    /**
     * 拼团价格
     */
    private BigDecimal teamPrice;
    /**
     * 商品价格
     */
    private BigDecimal productPrice;
    /**
     * 优惠力度
     */
    private Double discount;

    public static TeamDiscountPriceModel of(BigDecimal price, BigDecimal teamPrice) {
        return TeamDiscountPriceModel.builder()
                .productPrice(price)
                .teamPrice(teamPrice)
                .discount(price.subtract(teamPrice).divide(price,2, RoundingMode.HALF_EVEN).doubleValue())
                .build();
    }
}
