package com.muyu.activity.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.activity.service.*;
import com.muyu.group.domain.ActivitySkuInfo;
import com.muyu.group.domain.ActivityTeamInfo;
import com.muyu.group.domain.ActivityTeamProductSkuInfo;
import com.muyu.group.domain.model.*;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.activity.mapper.ActivityTeamInfoMapper;
import com.muyu.group.domain.req.GroupActivityInfoReq;
import com.muyu.product.cache.ProjectSkuCache;
import com.muyu.product.cache.datasource.ProjectInfoData;
import com.muyu.product.cache.datasource.ProjectSkuData;
import com.muyu.product.domain.ProjectInfo;
import com.muyu.product.domain.ProjectSkuInfo;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class ActivityTeamInfoServiceImpl extends ServiceImpl<ActivityTeamInfoMapper, ActivityTeamInfo> implements ActivityTeamInfoService {
    @Autowired
    private ActivityTeamProductSkuInfoService activityTeamProductSkuInfoService;
    @Autowired
    private ActivityTeamOpenInfoService activityTeamOpenInfoService;
    @Autowired
    private TeamStrategyExemptionHundredService teamStrategyExemptionHundredService;
    @Autowired
    private TeamStrategyExemptionOrdinaryService teamStrategyExemptionOrdinaryService;;
    @Autowired
    private TeamStrategyExemptionService teamStrategyExemptionService;
    @Override
    public TableDataInfo<ActivityTeamInfoListModel> query(ActivityTeamInfoListQueryModel model){
        LambdaQueryWrapper<ActivityTeamInfo> wrapper = new LambdaQueryWrapper<>();
        wrapper.like(StringUtils.isNotEmpty(model.getKeyWord()), ActivityTeamInfo::getName, model.getKeyWord());
        wrapper.like(StringUtils.isNotEmpty(model.getStatus()), ActivityTeamInfo::getStatus, model.getStatus());
        Page<ActivityTeamInfo> infoPage = this.page(model.buildPage(), wrapper);
        List<ActivityTeamInfo> activityTeamInfos = infoPage.getRecords();
        List<ActivityTeamInfoListModel> activityTeamInfoListModelList = activityTeamInfos.stream().map(activityTeamInfo -> ActivityTeamInfoListModel.infoBuild(activityTeamInfo,
                (activityTeamInfoListModelBuilder) -> {
                    Long teamInTeamId = activityTeamOpenInfoService.getTeamInTeamId(activityTeamInfo.getId());
                    Long teamOpenTeamId = activityTeamOpenInfoService.getTeamOpenTeamId(activityTeamInfo.getId());
                    TeamDiscountPriceModel disCountPrice = activityTeamProductSkuInfoService.getDisCountPrice(activityTeamInfo.getId());
                    TeamStockModel stock = activityTeamProductSkuInfoService.getStock(activityTeamInfo.getId());
                    return activityTeamInfoListModelBuilder.teamStock(stock.getTeamStock())
                            .teamPrice(disCountPrice.getTeamPrice())
                            .openTeamNumber(teamOpenTeamId)
                            .addTeamNumber(teamInTeamId)
                            .productPrice(disCountPrice.getProductPrice())
                            .attendNumber(teamInTeamId+teamOpenTeamId)
                            .remainStock(stock.getRemainStock())
                            .build();
                })).toList();
        TableDataInfo<ActivityTeamInfoListModel> modelTableDataInfo = new TableDataInfo<>();
        modelTableDataInfo.setTotal(infoPage.getTotal());
        modelTableDataInfo.setRows(activityTeamInfoListModelList);
        return modelTableDataInfo;
    }

    /**
     * 添加拼团活动详细信息
     * @param inBuild
     * @return
     */
    @Override
    public boolean seve(GroupActivityModel inBuild) {
        //添加策略返回主键
        //免单
        if (inBuild.getStrategyType().equals("免单")){
            Long noOrderPolicyId = teamStrategyExemptionService.saveStrategy(TeamStrategyExemptionModel.freeBuild(inBuild));
            if (null!=noOrderPolicyId){
                inBuild.setStrategyId(noOrderPolicyId);
            }
        }
        //百人
        if (inBuild.getStrategyType().equals("百人")){
            Long hundredManStrategyId =teamStrategyExemptionHundredService.saveHuandrea(TeamStrategyExemptionHundredModel.normalBuild(inBuild));
            if (null!=hundredManStrategyId){
                inBuild.setStrategyId(hundredManStrategyId);
            }
        }
        //普通
        if (inBuild.getStrategyType().equals("普通")){
          Long commonStrategyId=  teamStrategyExemptionOrdinaryService.saveOrdinary(TeamStrategyExemptionOrdinaryModel.techBuild(inBuild));
          if (null!=commonStrategyId){
              inBuild.setStrategyId(commonStrategyId);
          }
        }
        //添加商品拼团
        ActivityTeamInfo activityTeamInfo = ActivityTeamInfo.inBuild(inBuild);
        boolean save = this.save(activityTeamInfo);
        if (save){
            //添过规格
            List<ActivitySkuInfo> productSkuList = inBuild.getProjectSkuInfoList();
            ActivityTeamProductSkuInfo productSkuInfo = new ActivityTeamProductSkuInfo();
            productSkuList.forEach(activitySkuInfo -> {
                productSkuInfo.setTeamId(activityTeamInfo.getId());
                productSkuInfo.setProductSku(activitySkuInfo.getProductSku());
                productSkuInfo.setProductId(inBuild.getProductId());
                productSkuInfo.setTeamPrice(activitySkuInfo.getTeamPrice());
                productSkuInfo.setTeamStock(activitySkuInfo.getTeamStock());
            });
            activityTeamProductSkuInfoService.save(productSkuInfo);

            return true;
        }
        return false;
    }
};
