package com.muyu.activity.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.group.domain.ActivityTeamProductSkuInfo;
import com.muyu.group.domain.model.TeamDiscountPriceModel;
import com.muyu.group.domain.model.TeamStockModel;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.activity.mapper.ActivityTeamProductSkuInfoMapper;
import com.muyu.activity.service.ActivityTeamProductSkuInfoService;
import com.muyu.product.cache.ProjectSkuCache;
import com.muyu.product.domain.ProjectSkuInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;

@Service
public class ActivityTeamProductSkuInfoServiceImpl extends ServiceImpl<ActivityTeamProductSkuInfoMapper, ActivityTeamProductSkuInfo> implements ActivityTeamProductSkuInfoService {
    @Autowired
    private ProjectSkuCache projectSkuCache;

    /**
     * 根据活动id查询商品价格
     * @param teamId
     * @return
     */
    @Override
    public TeamDiscountPriceModel getDisCountPrice(Long teamId) {
        //根据活动id查询
        List<ActivityTeamProductSkuInfo> activityTeamProductSkuInfoList = this.ownedByAl(teamId);
        if (activityTeamProductSkuInfoList.size()>0){
            Optional<TeamDiscountPriceModel> discountPriceModel = activityTeamProductSkuInfoList.stream().map(activityTeamProductSkuInfo -> {
                ProjectSkuInfo projectSkuInfo = projectSkuCache.get(activityTeamProductSkuInfo.getProductId(), activityTeamProductSkuInfo.getProductSku());
                return TeamDiscountPriceModel.of(projectSkuInfo.getPrice(), activityTeamProductSkuInfo.getTeamPrice());
            }).min((o1, o2) -> Double.valueOf(o2.getDiscount() * 100 - o1.getDiscount() * 100).intValue());
            if (discountPriceModel.isEmpty()){
                throw new ServiceException("活动下没有这个商品");
            }
            return discountPriceModel.get();
        }
        return TeamDiscountPriceModel.builder()
                .productPrice(BigDecimal.valueOf(0.00))
                .teamPrice(BigDecimal.valueOf(0.00))
                .discount(0.00)
                .build();
    }

    /**
     * 根据活动id查询商品库存
     * @param teamId
     * @return
     */
    @Override
    public TeamStockModel getStock(Long teamId) {
        List<ActivityTeamProductSkuInfo> activityTeamProductSkuInfoList = this.ownedByAl(teamId);
        return TeamStockModel.builder()
                .teamStock(activityTeamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getTeamStock).reduce(0L,Long::sum))
                .remainStock(activityTeamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getRemainStock).reduce(0L,Long::sum))
                .build();
    }
}
